-- Disclaimer:
-- Dieser Quellcode ist als Vorlage oder als Ideengeber gedacht. Er kann frei und ohne 
-- Auflagen oder Einschrnkungen verwendet oder verndert werden.
-- Es wird keine Garantie bernommen, dass eine Funktionsfhigkeit mit aktuellen und 
-- zuknftigen API-Versionen besteht. Der Autor bernimmt daher keine direkte oder indirekte 
-- Verantwortung, wenn dieser Code gar nicht oder nur fehlerhaft ausgefhrt wird.
-- Fr Anregungen und Fragen steht der Autor gerne zur Verfgung.

-- Thorsten Kansy, www.dotnetconsulting.eu

-- In Datenbank wechseln
USE dotnetconsulting_TSQL2;
GO

-- Aufsteigend nummerieren
SELECT ROW_NUMBER() OVER (ORDER BY [ID]), * FROM [dbo].[Vertriebszahlen];

-- Absteigend nummerieren
SELECT ROW_NUMBER() OVER (ORDER BY [ID] DESC), * FROM [dbo].[Vertriebszahlen];

-- Nummerieren, pro Land separat
SELECT ROW_NUMBER() OVER (PARTITION BY [Land] ORDER BY [ID]), * FROM [dbo].[Vertriebszahlen];

-- Beispiel: Erste und letzte Zeile (pro Land) ermitteln, sortiert nach Umsatz
-- Der beste und der schlechteste Verkufer also
-- Fr den Filter wird eine Unterabfrage (oder CTE) bentigt

SELECT 
	[Name],
	CASE WHEN [First] = 1 THEN 'Top' ELSE 'Flop' END 'Bezeichnung',
	FORMAT([Umsatz], 'N2'),
	[Land]
FROM
(
	SELECT ROW_NUMBER() OVER (PARTITION BY [Land] ORDER BY [Umsatz]) AS 'Last',
		   ROW_NUMBER() OVER (PARTITION BY [Land] ORDER BY [Umsatz] DESC) AS 'First',
	* FROM [dbo].[Vertriebszahlen]
) [_] -- Dummy Alias
WHERE [First] = 1 OR [Last] = 1;

-- SQL Server 2022+
SELECT 
	[Name],
	CASE WHEN [First] = 1 THEN 'Top' ELSE 'Flop' END 'Bezeichnung',
	FORMAT([Umsatz], 'N2'),
	[Land]
FROM
(
	SELECT ROW_NUMBER() OVER win1 AS 'Last',
		   ROW_NUMBER() OVER win2 AS 'First',
	* FROM [dbo].[Vertriebszahlen]
	WINDOW win1 AS (PARTITION BY [Land] ORDER BY [Umsatz]),
		   win2 AS (PARTITION BY [Land] ORDER BY [Umsatz] DESC)

) [_] -- Dummy Alias
WHERE [First] = 1 OR [Last] = 1;