-- Disclaimer:
-- Dieser Quellcode ist als Vorlage oder als Ideengeber gedacht. Er kann frei und ohne 
-- Auflagen oder Einschrnkungen verwendet oder verndert werden.
-- Es wird keine Garantie bernommen, dass eine Funktionsfhigkeit mit aktuellen und 
-- zuknftigen API-Versionen besteht. Der Autor bernimmt daher keine direkte oder indirekte 
-- Verantwortung, wenn dieser Code gar nicht oder nur fehlerhaft ausgefhrt wird.
-- Fr Anregungen und Fragen steht der Autor gerne zur Verfgung.

-- Thorsten Kansy, www.dotnetconsulting.eu

-- In Datenbank wechseln
USE dotnetconsulting_TSQL2;
GO

-- Verkufer sortiert nach Name, mit ihrem jeweiligen Verkaufsrang (Weltweit)
SELECT DENSE_RANK() OVER (ORDER BY [Umsatz]) AS 'Gesamtrang',
	   * 
FROM [dbo].[Vertriebszahlen]
ORDER BY [Name];

-- Verkufer sortiert nach Name, mit ihrem jeweiligen Verkaufsrang
-- nun zustzlich pro Land
SELECT DENSE_RANK() OVER (ORDER BY [Umsatz]) AS 'Gesamtrang',
	   DENSE_RANK() OVER (PARTITION BY [Land] ORDER BY [Umsatz]) AS 'Landesrang',
	   * 
FROM [dbo].[Vertriebszahlen]
ORDER BY [Name];

-- Umsatz auf je 1000 (abgerundet)
-- CTE verwenden, um den Ausdruck nur einmal im Code zu haben
;WITH [cte]
AS
(
	SELECT FLOOR([Umsatz] / 1000) * 1000 AS [Umsatz],
		   [Name],
		   [Land]
	FROM [dbo].[Vertriebszahlen] 
)
SELECT DENSE_RANK() OVER (ORDER BY [Umsatz]) AS 'Gesamtrang',
	   DENSE_RANK() OVER (PARTITION BY [Land] ORDER BY [Umsatz]) AS 'Landesrang',
	   FORMAT([Umsatz], 'N0'),
	   [Name],
	   [Land]
FROM [cte];


-- SQL Server 2022, Umsatz auf je 1000 (abgerundet)
-- CTE verwenden, um den Ausdruck nur einmal im Code zu haben
;WITH [cte]
AS
(
	SELECT FLOOR([Umsatz] / 1000) * 1000 AS [Umsatz],
		   [Name],
		   [Land]
	FROM [dbo].[Vertriebszahlen] 
)
SELECT DENSE_RANK() OVER win1 AS 'Gesamtrang',
	   DENSE_RANK() OVER win2 AS 'Landesrang',
	   FORMAT([Umsatz], 'N0'),
	   [Name],
	   [Land]
FROM [cte]
WINDOW win1 AS (ORDER BY [Umsatz]),
	   win2 AS (PARTITION BY [Land] ORDER BY [Umsatz]);