-- Disclaimer
-- Dieser Quellcode ist als Vorlage oder als Ideengeber gedacht. Er kann frei und ohne 
-- Auflagen oder Einschrnkungen verwendet oder verndert werden.
-- Jedoch wird keine Garantie bernommen, dass eine Funktionsfhigkeit mit aktuellen und 
-- zuknftigen API-Versionen besteht. Der Autor bernimmt daher keine direkte oder indirekte 
-- Verantwortung, wenn dieser Code gar nicht oder nur fehlerhaft ausgefhrt wird.
-- Fr Anregungen und Fragen stehe ich jedoch gerne zur Verfgung.

-- Thorsten Kansy, www.dotnetconsulting.eu

-- Datenbank (erneut) anlegen und wechseln
:SETVAR dbname dotnetconsulting_TemporalTables
USE [master];
IF EXISTS (SELECT * FROM [sys].[databases] WHERE [name] = '$(dbname)')
BEGIN
	ALTER DATABASE [$(dbname)] SET SINGLE_USER WITH ROLLBACK IMMEDIATE;
	DROP DATABASE [$(dbname)];
	PRINT '''$(dbname)''-Datenbank gelscht';
END
GO
CREATE DATABASE [$(dbname)];
GO
USE [$(dbname)];
PRINT '''$(dbname)''-Datenbank erstellt und gewechselt';
GO

-- Temporal Table neu anlegen
CREATE TABLE dbo.Werte
(
       ID INT IDENTITY(1,1) NOT NULL,
       Wert1 NVARCHAR(10) NULL,
       Wert2 NVARCHAR(10) NULL,

       StartTime datetime2(7) GENERATED ALWAYS AS ROW START HIDDEN NOT NULL,
       EndTime datetime2(7) GENERATED ALWAYS AS ROW END HIDDEN NOT NULL,
       PERIOD FOR SYSTEM_TIME(StartTime, EndTime),

       CONSTRAINT Werte_PK PRIMARY KEY (ID)
)
WITH
( SYSTEM_VERSIONING = ON (HISTORY_TABLE = dbo.Werte_History) );
GO


-- Ein paar Zeilen einfgen, ndern und lschen
INSERT [dbo].[Werte] (Wert1, Wert2) VALUES ('A1','A2'), ('B1','B2'), ('C1','C2');
WAITFOR DELAY '0:0:10';
UPDATE [dbo].[Werte] SET Wert1 = 'A1_neu' WHERE ID = 1;
WAITFOR DELAY '0:0:10';
DELETE [dbo].[Werte] WHERE ID = 2;

-- Schauen, wie die Daten und die Historisierung ausschaut
SELECT * FROM dbo.Werte;
SELECT *, StartTime, EndTime FROM dbo.Werte;

SELECT * FROM dbo.Werte_History;

-- AS OF
SELECT * FROM dbo.Werte FOR SYSTEM_TIME AS OF '2021-10-11 10:16:44.9695048';

DECLARE @d DATETIME = '2021-10-05T12:34:10';
SELECT * FROM dbo.Werte FOR SYSTEM_TIME AS OF @d;
GO

-- ALL
SELECT *, StartTime, EndTime FROM dbo.Werte FOR SYSTEM_TIME ALL;
GO

-- FROM TO (Auschluss der oberen Grenze)
DECLARE @d1 DATETIME = '2016-01-25 18:21:24';
DECLARE @d2 DATETIME = '2016-01-25 18:21:24';

SELECT * FROM dbo.Werte FOR SYSTEM_TIME FROM @d1 TO @d2;
GO

-- BETWEEN AND (Einschlielich der oberen Grenze)
DECLARE @d1 DATETIME = '2016-01-25 18:21:24';
DECLARE @d2 DATETIME = '2016-01-25 18:21:24';

SELECT * FROM dbo.Werte FOR SYSTEM_TIME BETWEEN  @d1 AND @d2;
GO

-- CONTAINED IN (Nur historische Werte)
DECLARE @d1 DATETIME = '2016-01-25 18:21:24';
DECLARE @d2 DATETIME = '2016-01-25 18:21:24';

SELECT * FROM dbo.Werte FOR SYSTEM_TIME CONTAINED IN (@d1, @d2);
GO